clear;
global N K Cdirect Cinv;
global PROF_INIT PROFILES EPSILON T;

% ------------------
% user Input Section
% ------------------

% INPUT PARAMETERS - this is the payoff matrix, first for player 1, then 
% for player 2.

PI(:,:,1) = [13 40 96; ...
              6 99 38; ...
             14 20 64]; 
         
PI(:,:,2) = [60 53 26; ...
             52 56 68; ...
             72 74 96]; 

% rand('state',0);
% D = rand(5,5,2,100000);
% PI = D(:,:,:,69485);

% Notes:
% (a) To generate a random game, use "PI = rand(3,3,N);" instead.
% (b) One can choose any arbitrary number of actions for each player by 
% choosing the dimensions of the PI matrix. In particular, it does not have
% to be a square matrix.         
         
% INPUT PARAMETERS - Switching cost function and its inverse. Please make
% sure the direct function and the inverse match. The current code assumes
% that c(t) = t.

Cdirect = inline('t'); % cost function: c(t)
Cinv    = inline('v'); % inverse cost function

% -------------------------
% end of User Input Section
% -------------------------


% --------------
% Initialization
% --------------

K = size(PI(:,:,1)); % K(1) and K(2) are the number of actions for each player
N = 2;               % number of players (CANNOT BE CHANGED)
PROF_INIT = 0;
PROFILES = K(1)*K(2);   % number of possible profiles
OUTPUT = 1;             % 1 To display output
MAX_ITER = 2*K(1)*K(2); % initial maximal number of stages (gets increased dynamically)

% Changing the representation of the payoff matrix to be one-dimensional
% instead of two-dimensional.
DD = zeros(PROFILES,1,N);
for i = 1:K(1), for j = 1:K(2), DD(encodei([i j]),1,:) = PI(i,j,:); end; end;
D = squeeze(DD);
clear DD; 

% Computing 'EPSILON', which is used for breaking ties within the algorithm.
temp1 = sort(D(:,1));
temp2 = sort(D(:,2));
mindiff1 = Cinv(min(temp1(2:PROFILES)-temp1(1:PROFILES-1)));
mindiff2 = Cinv(min(temp2(2:PROFILES)-temp2(1:PROFILES-1)));
EPSILON  = 1e-14 + min([mindiff1 mindiff2])*1e-10;

% Computing 'T' to be big enough
maxdiff1 = Cinv(temp1(PROFILES)-temp1(1));
maxdiff2 = Cinv(temp2(PROFILES)-temp2(1));
T = 1 + max([maxdiff1 maxdiff2]);
clear mindiff1 mindiff2 maxdiff1 maxdiff2 temp1 temp2

% Allocating memory
t = zeros(MAX_ITER,1); % critical points in time
p = zeros(MAX_ITER,1); % player associated with each critical point in time      
V = zeros(PROFILES,N,MAX_ITER); % continuation values
F = zeros(PROFILES,N,MAX_ITER); % flags of immediate active moves: F(k,n,s) = 4 
                                % means that, during stage s, player n would like 
                                % to switch from profile k by choosing action 4.
                                % If it's equal 0, player n would like to stay put. 

EQUAL_T = 0;         % a red flag that gets turned on when a critical point for both players hits at the same time (non-generic)                                 
MULTIPLE_REAL = 0;   % a red flag that gets turned on when a critical point for the same player hits at the same time and affects 
                     % final values (non-generic)
MULTIPLE_NOTIMP = 0; % a red flag that gets turned on when there is an indifference point that does not affect values

% Initializing the first stage of the program (i.e., last stage of the game)
n    = 1;     % iteration counter
t(1) = T;     % critical time
p(1) = 0;     % player associated with stage
V(:,:,1) = D; % continuation values


% ---------------------------------------------------------------------------------
% Main loop of the algorithm
% ---------------------------------------------------------------------------------

while 1==1, 
   
   % reallocate memory if maximal number of stages reached     
   if n == MAX_ITER,
      MAX_ITER = MAX_ITER*2;
      VV = zeros(PROFILES,N,MAX_ITER);
      FF = zeros(PROFILES,N,MAX_ITER);
      tt = zeros(MAX_ITER,1);      
      pp = zeros(MAX_ITER,1);      
      
      VV(:,:,1:MAX_ITER/2) = V;
      FF(:,:,1:MAX_ITER/2) = F;
      tt(1:MAX_ITER/2) = t; 
      pp(1:MAX_ITER/2) = p;
      
      V = VV;
      F = FF;
      t = tt;
      p = pp;
      
      clear VV FF tt pp;
   end;      
   % end memory reallocation
   
   n = n+1; % stages counter
   [p(n) t(n) EQUAL_T] = FindNextStage(V(:,:,n-1),F(:,:,n-1));
   if EQUAL_T, break; end;  % Break if critical point hitting for both players at the same time (non-generic case) 

   disp([n t(n)]);
   
   % Break if reached the beginning of the game
   if t(n) == 0,
      n_bar = n-1;
      V = V(:,:,1:n_bar);
      F = F(:,:,1:n_bar);
      t = t(1:n_bar);
      p = p(1:n_bar);
      break;      
   end;
   
   % Update continuation values and beginning-of-stage equilibrium
   % strategies. Recall that n is the current stage and p(n) is the player
   % (1 or 2) associated with the n'th critical point in time
   
   tempF  = zeros(PROFILES,N);   % set of temporary active switches
   tempV  = zeros(PROFILES,N,N); % set of temporary continuation values, with the third dimension 'prioritizing' each player 
   tempV2 = -999*ones(PROFILES,N,N); % same, used to determine convergence

   % Update continuation values as of "just before" t(n). That is, if there are
   % switches, they are computed at time t(n)-EPSILON. EPSILON should be a
   % small positive number, constrained by computer precision.
   
   tempV(:,:,p(n)) = ComputeContValues(F(:,:,n-1),V(:,:,n-1),3-p(n),t(n)-EPSILON);
   tempp  = p(n);

   % Reupdate the set of switches that become 'active' due to change in 
   % incentives at t(n). This involves a loop over all profiles, until no
   % additional updates occur.
   
   while sum(sum(tempV(:,:,tempp) ~= tempV2(:,:,tempp))),
      tempV2(:,:,tempp) = tempV(:,:,tempp);
      for k = 1:PROFILES,
         prof = decodei(k);
         temp = zeros(K(tempp),1);
         for a = 1:K(tempp),
             temp(a) = tempV(Destination(k,tempp,a),tempp,tempp) - (Cdirect(t(n)-2*EPSILON))*(prof(tempp)~=a);  
         end;
         
         temp2   = max(temp);
         tempind = find(temp==temp2);
         if length(tempind) > 1,
            tempvalue = tempV(Destination(k,tempp,tempind(1)),3-tempp,tempp);
            for kk = 2:length(tempind),
               if tempvalue ~= tempV(Destination(k,tempp,tempind(kk)),3-tempp,tempp),
                   MULTIPLE_REAL = 1;
               end;
            end;
            MULTIPLE_NOTIMP = 1;
         end;    
                  
         [temp2 tempF(k,tempp)] = max(temp);
         
         prof = decodei(k);
         tempV(k,3-tempp,3-tempp) = tempV(Destination(k,tempp,tempF(k,tempp)),3-tempp,tempp);
         tempV(k,tempp,3-tempp)   = tempV(Destination(k,tempp,tempF(k,tempp)),tempp,tempp) ...
                                        - Cdirect(t(n)-EPSILON)*(prof(tempp)~=tempF(k,tempp));
                                     
         tempF(k,tempp) = tempF(k,tempp)*(tempF(k,tempp)~=prof(tempp));
      end;
      tempp = 3-tempp; % the other player
   end;
   
   % Active switches and continuation values through stage n
   
   F(:,:,n) = tempF;
   V(:,:,n) = tempV(:,:,tempp);
               
   % Check for termination (i.e., maximal number of active switches)
   
   if sum(sum(F(:,:,n)>0)) == (K(1)-1)*K(2) + (K(2)-1)*K(1),
      n_bar = n;
      V = V(:,:,1:n_bar);
      F = F(:,:,1:n_bar);
      t = t(1:n_bar);
      p = p(1:n_bar);
      t(n+1) = 0;
      break;
   end;   
   
end;   


% ---------------------------------------------------------------------------------
% Output
% ---------------------------------------------------------------------------------

% Tracing the equilibrium path from the equilibrium strategies
if EQUAL_T == 0,
   starting_nodes = F(:,1,n_bar)==0 & F(:,2,n_bar)==0;
   if sum(starting_nodes) > 1,
      if (max(V(starting_nodes,1,n_bar))==min(V(starting_nodes,1,n_bar)) & ...
          max(V(starting_nodes,2,n_bar))==min(V(starting_nodes,2,n_bar))),
         MULTIPLE_NOTIMP = 1;
      else,
         MULTIPLE_REAL = 1;
      end;
   end;
   eq(1) = min(find(sum(F(:,:,n_bar)')==0));
   switches = 0;
   for i = n_bar:-1:1,
      while (F(eq(switches+1),1,i)>0 | F(eq(switches+1),2,i)>0),
         if F(eq(switches+1),1,i) > 0,
            eq(switches+2) = Destination(eq(switches+1),1,F(eq(switches+1),1,i));
         elseif F(eq(switches+1),2,i) > 0,
            eq(switches+2) = Destination(eq(switches+1),2,F(eq(switches+1),2,i));
         end;   
         switches = switches + 1;
         tswitch(switches+1) = t(i+1);
      end;   
   end;   
end;
   
if EQUAL_T == 1,
   disp(sprintf('NON-GENERIC CASE: Equal critical times for both players hit at t = %3.3f --> No Order-Inavriance !!!',t(n)));
   flag = 2;
end;

% Output report
if OUTPUT & EQUAL_T == 0,
   disp('payoff matrix for player 1 is:');
   disp(PI(:,:,1));
   disp('payoff matrix for player 2 is:');
   disp(PI(:,:,2));
   disp(sprintf('final values are [%3.3f,%3.3f]',V(eq(1),:,n_bar)));
   disp(sprintf('initial profile is [%d,%d]',decodei(eq(1))));
   for i = 1:switches,
      disp(sprintf('switch %d occurs at time %3.3f into profile [%d,%d]',i,tswitch(i+1),decodei(eq(i+1))));
   end;
   if MULTIPLE_REAL == 1,
      disp('NOTE: This game is non-generic and has multiple equilibria that might have different equilibrium paths and values');       
   elseif MULTIPLE_NOTIMP == 1,
      disp('NOTE: This game has multiple equilibria that might have different equilibrium paths, but all of them have the same equilibrium values');       
   end;   
end;

% end program